@@ -28,6 +28,7 @@ arch_native = pmb.parse.arch.alpine_native()
 # (which may contain a vulnerable apk/openssl, and allows an attacker to
 # exploit the system!)
 apk_tools_min_version = {"edge": "2.14.0-r5",
+                         "v3.19": "2.14.0-r5",
                          "v3.18": "2.14.0-r2",
                          "v3.17": "2.12.10-r1",
                          "v3.16": "2.12.9-r3",
@@ -206,19 +207,21 @@ chroot_host_path = os.environ["PATH"] + ":/usr/sbin/"
 # $WORK gets replaced with args.work
 # $ARCH gets replaced with the chroot architecture (eg. x86_64, armhf)
 # $CHANNEL gets replaced with the release channel (e.g. edge, v21.03)
+# Use no more than one dir after /mnt/pmbootstrap, see remove_mnt_pmbootstrap.
 chroot_mount_bind = {
     "/proc": "/proc",
     "$WORK/cache_apk_$ARCH": "/var/cache/apk",
     "$WORK/cache_appstream/$ARCH/$CHANNEL": "/mnt/appstream-data",
-    "$WORK/cache_ccache_$ARCH": "/mnt/pmbootstrap-ccache",
+    "$WORK/cache_ccache_$ARCH": "/mnt/pmbootstrap/ccache",
     "$WORK/cache_distfiles": "/var/cache/distfiles",
-    "$WORK/cache_git": "/mnt/pmbootstrap-git",
-    "$WORK/cache_go": "/mnt/pmbootstrap-go",
-    "$WORK/cache_rust": "/mnt/pmbootstrap-rust",
-    "$WORK/config_abuild": "/mnt/pmbootstrap-abuild-config",
+    "$WORK/cache_git": "/mnt/pmbootstrap/git",
+    "$WORK/cache_go": "/mnt/pmbootstrap/go",
+    "$WORK/cache_rust": "/mnt/pmbootstrap/rust",
+    "$WORK/config_abuild": "/mnt/pmbootstrap/abuild-config",
     "$WORK/config_apk_keys": "/etc/apk/keys",
-    "$WORK/images_netboot": "/mnt/pmbootstrap-netboot",
-    "$WORK/packages/$CHANNEL": "/mnt/pmbootstrap-packages",
+    "$WORK/cache_sccache": "/mnt/pmbootstrap/sccache",
+    "$WORK/images_netboot": "/mnt/pmbootstrap/netboot",
+    "$WORK/packages/$CHANNEL": "/mnt/pmbootstrap/packages",
 }
 
 # Building chroots (all chroots, except for the rootfs_ chroot) get symlinks in
@@ -234,14 +237,15 @@ chroot_mount_bind = {
 # rust depends caching described above) and to cache build artifacts (GOCACHE,
 # similar to ccache).
 chroot_home_symlinks = {
-    "/mnt/pmbootstrap-abuild-config": "/home/pmos/.abuild",
-    "/mnt/pmbootstrap-ccache": "/home/pmos/.ccache",
-    "/mnt/pmbootstrap-packages": "/home/pmos/packages/pmos",
-    "/mnt/pmbootstrap-go/gocache": "/home/pmos/.cache/go-build",
-    "/mnt/pmbootstrap-go/gomodcache": "/home/pmos/go/pkg/mod",
-    "/mnt/pmbootstrap-rust/registry/index": "/home/pmos/.cargo/registry/index",
-    "/mnt/pmbootstrap-rust/registry/cache": "/home/pmos/.cargo/registry/cache",
-    "/mnt/pmbootstrap-rust/git/db": "/home/pmos/.cargo/git/db",
+    "/mnt/pmbootstrap/abuild-config": "/home/pmos/.abuild",
+    "/mnt/pmbootstrap/ccache": "/home/pmos/.ccache",
+    "/mnt/pmbootstrap/go/gocache": "/home/pmos/.cache/go-build",
+    "/mnt/pmbootstrap/go/gomodcache": "/home/pmos/go/pkg/mod",
+    "/mnt/pmbootstrap/packages": "/home/pmos/packages/pmos",
+    "/mnt/pmbootstrap/rust/git/db": "/home/pmos/.cargo/git/db",
+    "/mnt/pmbootstrap/rust/registry/cache": "/home/pmos/.cargo/registry/cache",
+    "/mnt/pmbootstrap/rust/registry/index": "/home/pmos/.cargo/registry/index",
+    "/mnt/pmbootstrap/sccache": "/home/pmos/.cache/sccache",
 }
 
 # Device nodes to be created in each chroot. Syntax for each entry:
@@ -267,8 +271,8 @@ chroot_outdated = 3600 * 24 * 2
 #
 # Officially supported host/target architectures for postmarketOS. Only
 # specify architectures supported by Alpine here. For cross-compiling,
-# we need to generate the "musl-$ARCH", "binutils-$ARCH" and "gcc-$ARCH"
-# packages (use "pmbootstrap aportgen musl-armhf" etc.).
+# we need to generate the "musl-$ARCH" and "gcc-$ARCH" packages (use
+# "pmbootstrap aportgen musl-armhf" etc.).
 build_device_architectures = ["armhf", "armv7", "aarch64", "x86_64", "x86", "riscv64"]
 
 # Packages that will be installed in a chroot before it builds packages
@@ -416,6 +420,7 @@ kconfig_options_iwd = {
             "PKCS7_MESSAGE_PARSER": True,
             "PKCS8_PRIVATE_KEY_PARSER": True,
             "X509_CERTIFICATE_PARSER": True,
+            "RFKILL": True,
         },
     },
 }
@@ -548,6 +553,11 @@ kconfig_options_containers = {
             "MACVLAN": True,  # Network Drivers
         }
     },
+    ">=3.13": {
+        "all": { # needed for iptables-nft (used by docker,tailscale)
+            "NFT_COMPAT": True,
+        }
+    },
     ">=3.14": {
         "all": {  # all arches
             "CGROUP_NET_PRIO": True,  # Optional section
@@ -635,7 +645,8 @@ kconfig_options_usb_gadgets = {
             "USB_MASS_STORAGE": False,
             "USB_G_SERIAL": False,
             # enable configfs gadgets
-            "USB_CONFIGFS_RNDIS": True,  # USB networking via RNDIS
+            "USB_CONFIGFS_NCM": True,  # USB networking via NCM
+            "USB_CONFIGFS_RNDIS": True,  # USB networking via RNDIS (legacy)
         },
     },
 }
@@ -644,10 +655,20 @@ kconfig_options_usb_gadgets = {
 kconfig_options_community = {
     ">=0.0.0": {  # all versions
         "all": {  # all arches
+            "BINFMT_MISC": True,  # register binary formats
+            "CIFS": True,  # mount SMB shares
             "INPUT_UINPUT": True,  # buffyboard
+            "LEDS_TRIGGER_PATTERN": True,  # feedbackd
             "LEDS_TRIGGER_TIMER": True,  # hfd-service
+            "NETFILTER_XT_MATCH_STATISTIC": True,  # kube-proxy
             "NETFILTER_XT_MATCH_TCPMSS": True,  # change MTU, e.g. for Wireguard
             "NETFILTER_XT_TARGET_TCPMSS": True,  # change MTU, e.g. for Wireguard
+            # TODO: Depends on SUSPEND which is not enabled for some devices
+            # "PM_WAKELOCKS": True,  # Sxmo
+            "SND_USB_AUDIO": True,  # USB audio devices
+            "UCLAMP_TASK": True,  # Scheduler hints
+            "UCLAMP_TASK_GROUP": True,  # Scheduler hints
+            "UHID": True,  # e.g. Bluetooth input devices
         },
     },
 }
@@ -664,7 +685,16 @@ kconfig_options_uefi = {
             "EFI_PARAMS_FROM_FDT": True,
             "EFI_RUNTIME_WRAPPERS": True,
             "EFI_GENERIC_STUB": True,
-        }
+        },
+        "x86_64": {
+            "EFI_MIXED": True,
+        },
+    },
+    ">=6.1.0": {
+        "aarch64": {
+            # Required EFI booting compressed kernels on this arch
+            "EFI_ZBOOT": True,
+        },
     },
 }
 
@@ -681,9 +711,9 @@ apkbuild_package_attributes = {
     "install": {"array": True},
     "triggers": {"array": True},
 
-    # UI meta-packages can specify apps in "_pmb_recommends" to be explicitly
-    # installed by default, and not implicitly as dependency of the UI meta-
-    # package ("depends"). This makes these apps uninstallable, without
+    # Packages can specify soft dependencies in "_pmb_recommends" to be
+    # explicitly installed by default, and not implicitly as a hard dependency
+    # of the package ("depends"). This makes these apps uninstallable, without
     # removing the meta-package. (#1933). To disable this feature, use:
     # "pmbootstrap install --no-recommends".
     "_pmb_recommends": {"array": True},
@@ -747,7 +777,7 @@ apkbuild_attributes = {
 # Reference: https://postmarketos.org/apkbuild-options
 apkbuild_custom_valid_options = [
     "!pmb:crossdirect",
-    "!pmb:kconfig-check",
+    "!pmb:kconfigcheck",
     "pmb:kconfigcheck-community",
     "pmb:kconfigcheck-containers",
     "pmb:kconfigcheck-iwd",
@@ -810,7 +840,9 @@ deviceinfo_attributes = [
     "generate_bootimg",
     "header_version",
     "bootimg_qcdt",
-    "bootimg_mtk_mkimage",
+    "bootimg_mtk_mkimage", # deprecated
+    "bootimg_mtk_label_kernel",
+    "bootimg_mtk_label_ramdisk",
     "bootimg_dtb_second",
     "bootimg_custom_args",
     "flash_offset_base",
@@ -873,9 +905,6 @@ default_ip = "172.16.42.1"
 install_native_packages = ["cryptsetup", "util-linux", "parted"]
 install_device_packages = ["postmarketos-base"]
 
-# Groups for the default user
-install_user_groups = ["wheel", "video", "audio", "input", "plugdev", "netdev"]
-
 #
 # FLASH
 #
@@ -984,20 +1013,23 @@ flashers = {
             "list_devices": [["heimdall", "detect"]],
             "flash_rootfs": [
                 ["heimdall_wait_for_device.sh"],
-                ["heimdall", "flash", "--$PARTITION_ROOTFS", "$IMAGE"]],
+                ["heimdall", "flash", "--$PARTITION_ROOTFS", "$IMAGE",
+                 "$NO_REBOOT", "$RESUME"]],
             "flash_kernel": [
                 ["heimdall_wait_for_device.sh"],
                 ["heimdall", "flash", "--$PARTITION_KERNEL",
-                 "$BOOT/boot.img$FLAVOR"]],
+                 "$BOOT/boot.img$FLAVOR", "$NO_REBOOT", "$RESUME"]],
             "flash_vbmeta": [
                 ["avbtool", "make_vbmeta_image", "--flags", "2",
                     "--padding_size", "$FLASH_PAGESIZE",
                     "--output", "/vbmeta.img"],
-                ["heimdall", "flash", "--$PARTITION_VBMETA", "/vbmeta.img"],
+                ["heimdall", "flash", "--$PARTITION_VBMETA", "/vbmeta.img",
+                 "$NO_REBOOT", "$RESUME"],
                 ["rm", "-f", "/vbmeta.img"]],
             "flash_lk2nd": [
                 ["heimdall_wait_for_device.sh"],
-                ["heimdall", "flash", "--$PARTITION_KERNEL", "$BOOT/lk2nd.img"]]
+                ["heimdall", "flash", "--$PARTITION_KERNEL", "$BOOT/lk2nd.img",
+                 "$NO_REBOOT", "$RESUME"]]
         },
     },
     "adb": {
@@ -1076,7 +1108,7 @@ git_repo_outdated = 3600 * 24 * 2
 #
 aportgen = {
     "cross": {
-        "prefixes": ["binutils", "busybox-static", "gcc", "musl", "grub-efi"],
+        "prefixes": ["busybox-static", "gcc", "musl", "grub-efi"],
         "confirm_overwrite": False,
     },
     "device/testing": {
@@ -1111,6 +1143,8 @@ newapkbuild_arguments_switches_pkgtypes = [
     ["-m", "meson", "create meson package (assume meson.build is there)"],
     ["-p", "perl", "create perl package (assume Makefile.PL is there)"],
     ["-y", "python", "create python package (assume setup.py is there)"],
+    ["-e", "python_gpep517", "create python package (assume pyproject.toml is there)"],
+    ["-r", "rust", "create rust package (assume Cargo.toml is there)"],
 ]
 newapkbuild_arguments_switches_other = [
     ["-s", "sourceforge", "use sourceforge source URL"],
