@@ -5,6 +5,7 @@ import os
 import re
 import glob
 import shlex
+import sys
 
 import pmb.chroot
 import pmb.chroot.apk
@@ -152,8 +153,10 @@ def create_home_from_skel(args):
     Create /home/{user} from /etc/skel
     """
     rootfs = args.work + "/chroot_native/mnt/install"
+    # In btrfs, home subvol & home dir is created in format.py
+    if args.filesystem != "btrfs":
+        pmb.helpers.run.root(args, ["mkdir", rootfs + "/home"])
     homedir = rootfs + "/home/" + args.user
-    pmb.helpers.run.root(args, ["mkdir", rootfs + "/home"])
     if os.path.exists(f"{rootfs}/etc/skel"):
         pmb.helpers.run.root(args, ["cp", "-a", f"{rootfs}/etc/skel", homedir])
     else:
@@ -165,7 +168,7 @@ def configure_apk(args):
     """
     Copy over all official keys, and the keys used to compile local packages
     (unless --no-local-pkgs is set). Then copy the corresponding APKINDEX files
-    and remove the /mnt/pmbootstrap-packages repository.
+    and remove the /mnt/pmbootstrap/packages repository.
     """
     # Official keys
     pattern = f"{pmb.config.apk_keys_path}/*.pub"
@@ -187,7 +190,7 @@ def configure_apk(args):
         pmb.helpers.run.root(args, ["cp", f, rootfs + "/var/cache/apk/"])
 
     # Disable pmbootstrap repository
-    pmb.helpers.run.root(args, ["sed", "-i", r"/\/mnt\/pmbootstrap-packages/d",
+    pmb.helpers.run.root(args, ["sed", "-i", r"/\/mnt\/pmbootstrap\/packages/d",
                                 rootfs + "/etc/apk/repositories"])
     pmb.helpers.run.user(args, ["cat", rootfs + "/etc/apk/repositories"])
 
@@ -204,7 +207,13 @@ def set_user(args):
     if not pmb.chroot.user_exists(args, args.user, suffix):
         pmb.chroot.root(args, ["adduser", "-D", "-u", "10000", args.user],
                         suffix)
-    groups = pmb.install.ui.get_groups(args) + pmb.config.install_user_groups
+
+    pmaports_cfg = pmb.config.pmaports.read_config(args)
+    groups = []
+    groups += pmaports_cfg.get("install_user_groups",
+                               "audio,input,netdev,plugdev,video,wheel").split(",")
+    groups += pmb.install.ui.get_groups(args)
+
     for group in groups:
         pmb.chroot.root(args, ["addgroup", "-S", group], suffix,
                         check=False)
@@ -408,12 +417,12 @@ def setup_appstream(args):
     if "alpine-appstream-downloader" not in installed_pkgs or args.offline:
         return
 
-    pmb.chroot.root(args, ["alpine-appstream-downloader",
-                           "/mnt/appstream-data"], suffix)
-    pmb.chroot.root(args, ["mkdir", "-p", "/var/lib/swcatalog"], suffix)
-    pmb.chroot.root(args, ["cp", "-r", "/mnt/appstream-data/icons",
-                           "/mnt/appstream-data/xml",
-                           "-t", "/var/lib/swcatalog"], suffix)
+    if not pmb.chroot.root(args, ["alpine-appstream-downloader",
+                                  "/mnt/appstream-data"], suffix, check=False):
+        pmb.chroot.root(args, ["mkdir", "-p", "/var/lib/swcatalog"], suffix)
+        pmb.chroot.root(args, ["cp", "-r", "/mnt/appstream-data/icons",
+                               "/mnt/appstream-data/xml",
+                               "-t", "/var/lib/swcatalog"], suffix)
 
 
 def disable_sshd(args):
@@ -615,20 +624,33 @@ def write_cgpt_kpart(args, layout, suffix):
         args, ["dd", f"if={filename}", f"of=/dev/installp{layout['kernel']}"])
 
 
-def sanity_check_sdcard(args):
-    device = args.sdcard
+def sanity_check_boot_size(args):
+    default = pmb.config.defaults["boot_size"]
+    if int(args.boot_size) >= int(default):
+        return
+    logging.error("ERROR: your pmbootstrap has a small boot_size of"
+                  f" {args.boot_size} configured, probably because the config"
+                  " has been created with an old version.")
+    logging.error("This can lead to problems later on, we recommend setting it"
+                  f" to {default} MiB.")
+    logging.error(f"Run 'pmbootstrap config boot_size {default}' and try again.")
+    sys.exit(1)
+
+
+def sanity_check_disk(args):
+    device = args.disk
     device_name = os.path.basename(device)
     if not os.path.exists(device):
-        raise RuntimeError(f"{device} doesn't exist, is the sdcard plugged?")
+        raise RuntimeError(f"{device} doesn't exist, is the disk plugged?")
     if os.path.isdir('/sys/class/block/{}'.format(device_name)):
         with open('/sys/class/block/{}/ro'.format(device_name), 'r') as handle:
             ro = handle.read()
         if ro == '1\n':
-            raise RuntimeError(f"{device} is read-only, is the sdcard locked?")
+            raise RuntimeError(f"{device} is read-only, maybe a locked SD card?")
 
 
-def sanity_check_sdcard_size(args):
-    device = args.sdcard
+def sanity_check_disk_size(args):
+    device = args.disk
     devpath = os.path.realpath(device)
     sysfs = '/sys/class/block/{}/size'.format(devpath.replace('/dev/', ''))
     if not os.path.isfile(sysfs):
@@ -643,7 +665,7 @@ def sanity_check_sdcard_size(args):
     human = "{:.2f} GiB".format(size / 2 / 1024 / 1024)
 
     # Warn if the size is larger than 100GiB
-    if size > (100 * 2 * 1024 * 1024):
+    if not args.assume_yes and size > (100 * 2 * 1024 * 1024):
         if not pmb.helpers.cli.confirm(args,
                                        f"WARNING: The target disk ({devpath}) "
                                        "is larger than a usual SD card "
@@ -751,7 +773,22 @@ def create_fstab(args, layout, suffix):
     boot_filesystem = args.deviceinfo["boot_filesystem"] or "ext2"
     root_filesystem = pmb.install.get_root_filesystem(args)
 
-    fstab = f"""
+    if root_filesystem == "btrfs":
+        # btrfs gets separate subvolumes for root, var and home
+        fstab = f"""
+# <file system> <mount point> <type> <options> <dump> <pass>
+{root_mount_point} / btrfs subvol=@,compress=zstd:2,ssd 0 0
+{root_mount_point} /home btrfs subvol=@home,compress=zstd:2,ssd 0 0
+{root_mount_point} /root btrfs subvol=@root,compress=zstd:2,ssd 0 0
+{root_mount_point} /srv btrfs subvol=@srv,compress=zstd:2,ssd 0 0
+{root_mount_point} /var btrfs subvol=@var,ssd 0 0
+{root_mount_point} /.snapshots btrfs subvol=@snapshots,compress=zstd:2,ssd 0 0
+
+{boot_mount_point} /boot {boot_filesystem} defaults 0 0
+""".lstrip()
+
+    else:
+        fstab = f"""
 # <file system> <mount point> <type> <options> <dump> <pass>
 {root_mount_point} / {root_filesystem} defaults 0 0
 {boot_mount_point} /boot {boot_filesystem} defaults 0 0
@@ -764,7 +801,7 @@ def create_fstab(args, layout, suffix):
 
 def install_system_image(args, size_reserve, suffix, step, steps,
                          boot_label="pmOS_boot", root_label="pmOS_root",
-                         split=False, sdcard=None):
+                         split=False, disk=None):
     """
     :param size_reserve: empty partition between root and boot in MiB (pma#463)
     :param suffix: the chroot suffix, where the rootfs that will be installed
@@ -774,9 +811,9 @@ def install_system_image(args, size_reserve, suffix, step, steps,
     :param boot_label: label of the boot partition (e.g. "pmOS_boot")
     :param root_label: label of the root partition (e.g. "pmOS_root")
     :param split: create separate images for boot and root partitions
-    :param sdcard: path to sdcard device (e.g. /dev/mmcblk0) or None
+    :param disk: path to disk block device (e.g. /dev/mmcblk0) or None
     """
-    # Partition and fill image/sdcard
+    # Partition and fill image file/disk block device
     logging.info(f"*** ({step}/{steps}) PREPARE INSTALL BLOCKDEVICE ***")
     pmb.chroot.shutdown(args, True)
     (size_boot, size_root) = get_subpartitions_size(args, suffix)
@@ -784,7 +821,7 @@ def install_system_image(args, size_reserve, suffix, step, steps,
              and args.install_cgpt)
     if not args.rsync:
         pmb.install.blockdevice.create(args, size_boot, size_root,
-                                       size_reserve, split, sdcard)
+                                       size_reserve, split, disk)
         if not split:
             if args.deviceinfo["cgpt_kpart"] and args.install_cgpt:
                 pmb.install.partition_cgpt(
@@ -792,9 +829,9 @@ def install_system_image(args, size_reserve, suffix, step, steps,
             else:
                 pmb.install.partition(args, layout, size_boot, size_reserve)
     if not split:
-        pmb.install.partitions_mount(args, layout, sdcard)
+        pmb.install.partitions_mount(args, layout, disk)
 
-    pmb.install.format(args, layout, boot_label, root_label, sdcard)
+    pmb.install.format(args, layout, boot_label, root_label, disk)
 
     # Create /etc/fstab and /etc/crypttab
     logging.info("(native) create /etc/fstab")
@@ -810,6 +847,7 @@ def install_system_image(args, size_reserve, suffix, step, steps,
     # Clean up after running mkinitfs in chroot
     pmb.helpers.mount.umount_all(args, f"{args.work}/chroot_{suffix}")
     pmb.helpers.run.root(args, ["rm", f"{args.work}/chroot_{suffix}/in-pmbootstrap"])
+    pmb.chroot.remove_mnt_pmbootstrap(args, suffix)
 
     # Just copy all the files
     logging.info(f"*** ({step + 1}/{steps}) FILL INSTALL BLOCKDEVICE ***")
@@ -824,8 +862,8 @@ def install_system_image(args, size_reserve, suffix, step, steps,
         embed_firmware(args, suffix)
         write_cgpt_kpart(args, layout, suffix)
 
-    if sdcard:
-        logging.info("Unmounting SD card (this may take a while "
+    if disk:
+        logging.info(f"Unmounting disk {disk} (this may take a while "
                      "to sync, please wait)")
     pmb.chroot.shutdown(args, True)
 
@@ -834,7 +872,7 @@ def install_system_image(args, size_reserve, suffix, step, steps,
     if sparse is None:
         sparse = args.deviceinfo["flash_sparse"] == "true"
 
-    if sparse and not split and not sdcard:
+    if sparse and not split and not disk:
         logging.info("(native) make sparse rootfs")
         pmb.chroot.apk.install(args, ["android-tools"])
         sys_image = args.device + ".img"
@@ -880,7 +918,7 @@ def print_flash_info(args):
     logging.info("Run the following to flash your installation to the"
                  " target device:")
 
-    if "flash_rootfs" in flasher_actions and not args.sdcard and \
+    if "flash_rootfs" in flasher_actions and not args.disk and \
             bool(args.split) == requires_split:
         logging.info("* pmbootstrap flasher flash_rootfs")
         logging.info("  Flashes the generated rootfs image to your device:")
@@ -913,8 +951,8 @@ def print_flash_info(args):
 
     # Most flash methods operate independently of the boot partition.
     # (e.g. an Android boot image is generated). In that case, "flash_kernel"
-    # works even when partitions are split or installing for sdcard.
-    # This is not possible if the flash method requires split partitions.
+    # works even when partitions are split or installing to disk. This is not
+    # possible if the flash method requires split partitions.
     if "flash_kernel" in flasher_actions and \
             (not requires_split or args.split):
         logging.info("* pmbootstrap flasher flash_kernel")
@@ -1026,7 +1064,7 @@ def install_on_device_installer(args, step, steps):
     boot_label = pmaports_cfg.get("supported_install_boot_label",
                                   "pmOS_inst_boot")
     install_system_image(args, size_reserve, suffix_installer, step, steps,
-                         boot_label, "pmOS_install", args.split, args.sdcard)
+                         boot_label, "pmOS_install", args.split, args.disk)
 
 
 def get_selected_providers(args, packages):
@@ -1047,6 +1085,56 @@ def get_selected_providers(args, packages):
     return providers
 
 
+def get_recommends(args, packages):
+    """
+    Look through the specified packages and collect additional packages
+    specified under _pmb_recommends in them. This is recursive, so it will dive
+    into packages that are listed under recommends to collect any packages they
+    might also have listed under their own _pmb_recommends.
+
+    Recursion is only done into packages found in pmaports.
+
+    If running with pmbootstrap install --no-recommends, this function returns
+    an empty list.
+
+    :returns: list of pkgnames, e.g. ["chatty", "gnome-contacts"]
+    """
+    ret = []
+    if not args.install_recommends:
+        return ret
+
+    for package in packages:
+        # Note that this ignores packages that don't exist. This means they
+        # aren't in pmaports. This is fine, with the assumption that
+        # installation will fail later in some other method if they truly don't
+        # exist in any repo.
+        apkbuild = pmb.helpers.pmaports.get(args, package, must_exist=False)
+        if not apkbuild:
+            continue
+        if package in apkbuild["subpackages"]:
+            # Just focus on the subpackage
+            apkbuild = apkbuild["subpackages"][package]
+            # The subpackage is None if the subpackage does not have a function
+            # in the APKBUILD (uses the default function), e.g. for most openrc
+            # subpackages. See pmb.parse._apkbuild._parse_subpackage().
+            if not apkbuild:
+                continue
+        recommends = apkbuild["_pmb_recommends"]
+        if recommends:
+            logging.debug(f"{package}: install _pmb_recommends:"
+                          f" {', '.join(recommends)}")
+            ret += recommends
+            # Call recursively in case recommends have pmb_recommends of their
+            # own.
+            ret += get_recommends(args, recommends)
+        # Also iterate through dependencies to collect any recommends they have
+        depends = apkbuild["depends"]
+        if depends:
+            ret += get_recommends(args, depends)
+
+    return ret
+
+
 def create_device_rootfs(args, step, steps):
     # List all packages to be installed (including the ones specified by --add)
     # and upgrade the installed packages/apkindexes
@@ -1075,8 +1163,6 @@ def create_device_rootfs(args, step, steps):
     if args.ui.lower() != "none":
         if args.ui_extras:
             install_packages += ["postmarketos-ui-" + args.ui + "-extras"]
-        if args.install_recommends:
-            install_packages += pmb.install.ui.get_recommends(args)
     if args.extra_packages.lower() != "none":
         install_packages += args.extra_packages.split(",")
     if args.add:
@@ -1105,6 +1191,9 @@ def create_device_rootfs(args, step, steps):
 
     pmb.helpers.repo.update(args, args.deviceinfo["arch"])
 
+    # Install uninstallable "dependencies" by default
+    install_packages += get_recommends(args, install_packages)
+
     # Explicitly call build on the install packages, to re-build them or any
     # dependency, in case the version increased
     if args.build_pkgs_on_install:
@@ -1147,9 +1236,10 @@ def create_device_rootfs(args, step, steps):
 
 def install(args):
     # Sanity checks
-    if not args.android_recovery_zip and args.sdcard:
-        sanity_check_sdcard(args)
-        sanity_check_sdcard_size(args)
+    sanity_check_boot_size(args)
+    if not args.android_recovery_zip and args.disk:
+        sanity_check_disk(args)
+        sanity_check_disk_size(args)
     if args.on_device_installer:
         sanity_check_ondev_version(args)
 
@@ -1163,6 +1253,9 @@ def install(args):
     else:
         steps = 4
 
+    if args.zap:
+        pmb.chroot.zap(args, False)
+
     # Install required programs in native chroot
     step = 1
     logging.info(f"*** ({step}/{steps}) PREPARE NATIVE CHROOT ***")
@@ -1184,7 +1277,7 @@ def install(args):
         install_on_device_installer(args, step, steps)
     else:
         install_system_image(args, 0, f"rootfs_{args.device}", step, steps,
-                             split=args.split, sdcard=args.sdcard)
+                             split=args.split, disk=args.disk)
 
     print_flash_info(args)
     print_sshd_info(args)
